#!/usr/bin/perl
#
# Program check_ora_table_space
# Written by: Erwan Arzur (erwan@netvalue.com)
# Updated by: Aaron Bostick (aaron_bostick@yahoo.com)
# License: GPL
#
# Last Modified: $Date: 2002/03/14 13:23:29 $
# Revisiin: $Revision: 1.2 $
#
# "check_ora_table_space.pl" plugin to check the state of Oracle 
# table spaces. Scarce documentation.
#
# you need DBD-Oracle-1.03.tar.gz and DBI-1.13.tar.gz from CPAN.org as
# well as some Oracle client stuff to use it.
#
# The SQL request comes from www.dbasupport.com
#

use DBI;
$ENV{"ORACLE_HOME"}="/home/oinstall/OraHome1";

my $dbuser = shift || &usage ("no user specified");
my $dbpass = shift || &usage ("no password specified");
my $tablespace = shift || &usage ("no table space  specified");
my $warnpct = int(shift) || &usage ("no warning state percentage specified");
my $critpct =  int(shift) || &usage ("no critical state percentage specified");

my $dbh = DBI->connect(	"dbi:Oracle:", $dbuser, $dbpass, { PrintError => 0, AutoCommit => 1, RaiseError => 0 } )
	|| &error ("cannot connect to $dbname: $DBI::errstr");

my $exit_code = -1;
$sth = $dbh->prepare(<<EOF
SELECT a.TABLESPACE_NAME, a.total, NVL(b.used,0) USED
FROM (SElECT TABLESPACE_NAME, SUM(bytes) total 
      FROM sys.dba_data_files GROUP BY TABLESPACE_NAME) a, 
     (SELECT TABLESPACE_NAME, SUM(bytes) used FROM dba_segments GROUP BY TABLESPACE_NAME) b 
      WHERE a.TABLESPACE_NAME='$tablespace' AND a.TABLESPACE_NAME=b.TABLESPACE_NAME(+)
EOF
)
	|| &error("Cannot prepare request : $DBI::errstr");
$sth->execute 
	|| &error("Cannot execute request : $DBI::errstr");

if (($tname, $tsize, $tused) = $sth->fetchrow)
{
	$tsize = int($tsize) / (1024*1024);
	$tused = int($tused) / (1024*1024);
	$pct_used = ($tused / $tsize) * 100;

	if ($pct_used > $warnpct) {
		if ($pct_used > $critpct) {
			$exit_code = 2
		} else {
			$exit_code = 1;
		}
	} else {
		$exit_code = 0;	
	}
	printf("Size: %0.1fMB  Used: %0.1fMB  Percent Used: %0.1f%\n", $tsize, $tused, $pct_used);
}
else {
	&error("CRITICAL: No row returned from query.");
}
$sth->finish
	|| &error ("Cannot finish database statement : $sth->errstr");
$dbh->disconnect 
	|| &error ("Cannot disconnect from database : $dbh->errstr");

exit ($exit_code);

sub usage {
	print "@_\n" if @_;
	print "usage : check_ora_table_space.pl <user\@sid> <passwd> <tablespace> <pctwarn> <pctcrit>\n";
	exit (-1);
}

sub error {
	print "@_\n" if @_;
	exit (2);
}
